@@ -15,9 +15,9 @@ Example Usage
         # for the purposes of testing. Remove this argument
         # or set it to None to make global commands be
         # properly global - note that they take 1 hour to
-        # propagate. Useful because commands are
-        # re-registered every time the bot starts.
-        debug_guild=7293012031203012
+        # propagate. Useful because commands have to be
+        # re-registered if their API definitions are changed.
+        debug_guild=staging_guild_id
     )
 
     msg_opt = slash.Option(
@@ -33,7 +33,8 @@ Example Usage
         message: msg_opt
     ):
         """Send a message in the bot's name""" # description of command
-        await ctx.respond(message, # respond to the interaction
+        # respond to the interaction, must be done within 3 seconds
+        await ctx.respond(message, # string (or str()able) message
             # sends a message without showing the command invocation
             rtype=slash.InteractionResponseType.ChannelMessage)
 
@@ -54,13 +55,14 @@ See the wiki_.
 from __future__ import annotations
 from enum import IntEnum
 from typing import Coroutine, Union
+from functools import partial
 import logging
 import datetime
 import asyncio
 import discord
 from discord.ext import commands
 
-__version__ = '0.1.0'
+__version__ = '0.2.0'
 
 class ApplicationCommandOptionType(IntEnum):
     """Possible option types. Default is ``STRING``."""
@@ -87,7 +89,8 @@ class InteractionResponseType(IntEnum):
     .. attribute:: ChannelMessage
 
         Respond with a message, but don't show user input.
-        Probably best suited for admin commands
+        Probably best suited for admin commands,
+        or commands with ephemeral responses.
     .. attribute:: ChannelMessageWithSource
 
         Show user input and send a message. Default.
@@ -107,6 +110,18 @@ class InteractionResponseType(IntEnum):
     # ACK a command without sending a message, showing the user's input
     AcknowledgeWithSource = 5
 
+class MessageFlags(IntEnum):
+    """Flags to pass to the ``flags`` argument of the interaction response.
+    See also: https://discord.dev/resources/channel#message-object-message-flags
+    """
+    CROSSPOSTED = 1 << 0
+    IS_CROSSPOST = 1 << 1
+    SUPPRESS_EMBEDS = 1 << 2
+    SOURCE_MESSAGE_DELETED = 1 << 3
+    URGENT = 1 << 4
+    HAS_THREAD = 1 << 5
+    EPHEMERAL = 1 << 6
+
 class _Route(discord.http.Route):
     BASE = 'https://discord.com/api/v8'
 
@@ -150,31 +165,29 @@ class Context(discord.Object, _AsyncInit):
         Webhook used for sending followup messages.
         None until interaction response has been sent
     """
-    cog = None # not a thing, but some things check for it
 
     async def __init__(self, client: SlashBot, cmd: Command, event: dict):
         self.client = client
         self.command = cmd
-        self.id = event['id']
+        self.id = int(event['id'])
         try:
-            self.guild = await self.client.fetch_guild(event['guild_id'])
+            self.guild = await self.client.fetch_guild(int(event['guild_id']))
         except discord.HTTPException:
             self.guild = discord.Object(event['guild_id'])
             logger.debug('Fetching guild %s for interaction %s failed',
-                         self.guild.id, self.id, exc_info=True)
+                         self.guild.id, self.id)
         try:
-            self.channel = await self.client.fetch_channel(event['channel_id'])
+            self.channel = await self.client.fetch_channel(int(event['channel_id']))
         except discord.HTTPException:
             self.channel = discord.Object(event['channel_id'])
             logger.debug('Fetching channel %s for interaction %s failed',
-                         self.channel.id, self.id, exc_info=True)
+                         self.channel.id, self.id)
         try:
-            self.author = await self.guild.fetch_member(event['member']['user']['id'])
+            self.author = await self.guild.fetch_member(int(event['member']['user']['id']))
         except (discord.HTTPException, AttributeError):
             self.author = discord.Member(
                 data=event['member'], guild=self.guild, state=self.client._connection)
-            logger.debug('Fetching member for interaction %s failed',
-                         self.id, exc_info=True)
+            logger.debug('Fetching member for interaction %s failed', self.id)
         self.token = event['token']
         # construct options into function-friendly form
         await self._kwargs_from_options(event['data'].get('options', []))
@@ -184,45 +197,84 @@ class Context(discord.Object, _AsyncInit):
             self.me = None
             logger.debug('Fetching member %s (me) in guild %s '
                          'for interaction %s failed',
-                         self.client.user.id, self.guild.id,
-                         self.id, exc_info=True)
+                         self.client.user.id, self.guild.id, self.id)
         self.webhook = None
 
     async def _kwargs_from_options(self, options):
+        self.cog = self.command.cog
         kwargs = {}
         for opt in options:
             if 'value' in opt:
                 value = opt['value']
+                for k, v in self.command.options.items():
+                    if v.name == opt['name']:
+                        opt['name'] = k
+                        break
+                else:
+                    raise commands.CommandInvokeError(
+                        f'No such option: {opt["name"]!r}')
                 opttype = self.command.options[opt['name']].type
                 if opttype == ApplicationCommandOptionType.USER:
-                    value = await self.guild.fetch_member(int(value))
+                    try:
+                        value = await self.guild.fetch_member(int(value))
+                    except (discord.HTTPException, AttributeError):
+                        value = discord.Object(value)
+                        logger.debug('Fetching member %s for interaction %s failed',
+                                     value, self.id)
                 elif opttype == ApplicationCommandOptionType.CHANNEL:
-                    value = await self.guild.fetch_channel(int(value))
+                    try:
+                        value = await self.client.fetch_channel(int(value))
+                    except (discord.HTTPException, AttributeError):
+                        value = discord.Object(value)
+                        logger.debug('Fetching channel %s for interaction %s failed',
+                                     value, self.id)
                 elif opttype == ApplicationCommandOptionType.ROLE:
                     value = self.guild.get_role(int(value))
+                    if value is None:
+                        value = discord.Object(value)
+                        logger.debug('Getting role %s for interaction %s failed',
+                                     value, self.id)
                 kwargs[opt['name']] = value
             elif 'options' in opt:
                 self.command = self.command.slash[opt['name']]
                 await self._kwargs_from_options(opt['options'])
                 return
-        kwargs[self.command._ctx_arg] = self
-        self.options = kwargs
+        if isinstance(self.command, Command):
+            kwargs[self.command._ctx_arg] = self
+            self.options = kwargs
+        elif isinstance(self.command, Group):
+            self.command = self.command.slash[opt['name']]
+            await self._kwargs_from_options(opt.get('options', []))
 
-    async def respond(self, content='', *, embeds=None, allowed_mentions=None,
-                      rtype=InteractionResponseType.ChannelMessageWithSource):
+    def __repr__(self):
+        return f'<Interaction id={self.id}>'
+
+    async def respond(
+        self, content='', *, embed=None, embeds=None, allowed_mentions=None,
+        flags=None, rtype=InteractionResponseType.ChannelMessageWithSource
+    ):
         """Respond to the interaction. If called again, edits the response.
 
         Parameters
         -----------
         content: :class:`str`
             The content of the message.
+        embed: :class:`discord.Embed`
+            Shorthand for respond(embeds=[embed])
         embeds: Iterable[:class:`discord.Embed`]
             Up to 10 embeds (any more will be silently discarded)
         allowed_mentions: :class:`discord.AllowedMentions`
             Mirrors normal ``allowed_mentions`` in Messageable.send
+        flags: Union[int, :class:`MessageFlags`]
+            Message flags, ORed together
         rtype: :class:`InteractionResponseType`
             The type of response to send. See that class's documentation.
         """
+        content = str(content)
+        if embed and embeds:
+            raise TypeError('Cannot specify both embed and embeds')
+        if embed:
+            embeds = [embed]
         if embeds:
             embeds = [emb.to_dict() for emb, _ in zip(embeds, range(10))]
         mentions = self.client.allowed_mentions
@@ -254,7 +306,9 @@ class Context(discord.Object, _AsyncInit):
             if embeds:
                 data['data']['embeds'] = embeds
             if mentions is not None:
-                data['allowed_mentions'] = mentions.to_dict()
+                data['data']['allowed_mentions'] = mentions.to_dict()
+            if flags and 'data' in data:
+                data['data']['flags'] = int(flags)
             path = f"/interactions/{self.id}/{self.token}/callback"
             route = _Route('POST', path, channel_id=self.channel.id,
                            guild_id=self.guild.id)
@@ -295,8 +349,7 @@ class Option:
     name: Optional[:class:`str`]
         The name of the option, if different from its argument name.
     default: Optional[:class:`bool`]
-        If ``True``, this is the first ``required`` option to complete.
-        Only one option can be ``default``. This defaults to ``False``.
+        No idea what this does. It doesn't work anyway.
     required: Optional[:class:`bool`]
         If ``True``, this option must be specified for a valid command
         invocation. Defaults to ``False``.
@@ -319,6 +372,11 @@ class Option:
         if self.choices is not None:
             self.choices = [Choice.from_data(c) for c in self.choices]
 
+    def __repr__(self):
+        return ("Option(name={0.name!r}, type='{0.type!s}', description=..., "
+                'default={0.default}, required={0.required}, '
+                'choices={1})').format(self, '[...]' if self.choices else '[]')
+
     def to_dict(self):
         data = {
             'type': int(self.type),
@@ -343,13 +401,16 @@ class Choice:
     -----------
     name: :class:`str`
         The description of the choice, displayed to users.
-    value: Union[:class:`str`, :class:`int`]
+    value: :class:`str`
         The actual value fed into the application.
     """
-    def __init__(self, name: str, value: Union[str, int]):
+    def __init__(self, name: str, value: str):
         self.name = name
         self.value = value
 
+    def __repr__(self):
+        return f'Choice(name={self.name!r}, value={self.value!r})'
+
     @classmethod
     def from_data(cls, data):
         if isinstance(data, cls):
@@ -366,6 +427,8 @@ class Command:
 
     Attributes
     -----------
+    coro: Coroutine
+        (Required) Original callback for the command.
     id: Optional[:class:`int`]
         ID of registered command. Can be None when not yet registered,
         or if not a top-level command.
@@ -378,18 +441,21 @@ class Command:
     parent: Optional[:class:`Group`]
         Parent (sub)command group.
     options: Mapping[:class:`str`, :class:`Option`]
-        Options for this command.
-    coro: Coroutine
-        Original callback for the command.
+        Options for this command. Not passable as a parameter;
+        can only be set by inspecting the function annotations.
     check: Coroutine
         Callback that prevents command execution
         if it returns False (not falsy, False).
     """
+    cog = None
+
     def __init__(self, coro: Coroutine, **kwargs):
         self.id = None
         self.name = kwargs.pop('name', coro.__name__)
         self.description = kwargs.pop('description', coro.__doc__)
         self.guild_id = kwargs.pop('guild', None)
+        if self.guild_id is not None:
+            self.guild_id = int(self.guild_id)
         self.parent = kwargs.pop('parent', None)
         self._ctx_arg = None
         self.options = {}
@@ -415,6 +481,9 @@ class Command:
             return self.name
         return self.parent.qualname + ' ' + self.name
 
+    def __str__(self):
+        return self.qualname
+
     def __hash__(self):
         return hash((self.name, self.guild_id))
 
@@ -428,21 +497,17 @@ class Command:
         return data
 
     async def invoke(self, ctx):
-        parents = [] # highest level parent last
-        parent = self.parent
-        while parent is not None:
-            parents.append(parent.coro)
-            parent = parent.parent
-        parents.extend(ctx.client._checks)
-        parents.append(self._check)
-        parents.reverse()
-        for check in parents:
-            if await check(ctx) is False:
-                return
+        if not await self.can_run(ctx):
+            raise commands.CheckFailure(
+                f'The check functions for {self.qualname} failed.')
         logger.debug('User %s running, in guild %s channel %s, command: %s',
                      ctx.author.id, ctx.guild.id, ctx.channel.id,
                      ctx.command.qualname)
-        await self.coro(**ctx.options)
+        await self.invoke_parents(ctx)
+        if self.cog is not None:
+            await self.coro(self.cog, **ctx.options)
+        else:
+            await self.coro(**ctx.options)
 
     def check(self, coro):
         """Set this command's check to this coroutine.
@@ -450,72 +515,95 @@ class Command:
         """
         self._check = coro
 
-class Group:
+    async def can_run(self, ctx):
+        parents = []  # highest level parent last
+        cogs = []
+        parent = self.parent
+        while parent is not None:
+            if parent.cog is not None:
+                if hasattr(parent.cog, 'cog_check'):
+                    if parent.cog.cog_check not in cogs:
+                        cogs.append(parent.cog.cog_check)
+                parents.append(partial(parent._check, parent.cog))
+            else:
+                parents.append(parent._check)
+            parent = parent.parent
+        parents.extend(cogs)
+        parents.extend(ctx.client._checks)
+        parents.reverse()  # highest level parent first
+        parents.append(self._check)
+        for check in parents:
+            if await check(ctx) is False:
+                return False
+        return True
+
+    async def invoke_parents(self, ctx):
+        parents = []
+        parent = self.parent
+        while parent is not None:
+            if not parent.in_addition:
+                parent = parent.parent
+                continue
+            if parent.cog is not None:
+                parents.append(partial(parent.coro, parent.cog))
+            else:
+                parents.append(parent.coro)
+            parent = parent.parent
+        parents.reverse()
+        for coro in parents:
+            await coro(ctx)
+
+class Group(Command):
     """Represents a group of slash commands.
+    Attributes are the same unless documented below.
 
     Attributes
     -----------
-    id: Optional[:class:`int`]
-        ID of registered group. Can be None when not yet registered,
-        or if not a top-level group.
-    name: :class:`str`
-        (Sub)command group name. Defaults to coro name.
-    description: :class:`str`
-        Description shown in command list. Defaults to coro doc.
-    guild_id: Optional[:class:`int`]
-        If present, this group only exists in this guild.
-    parent: Optional[:class:`Group`]
-        Parent command group.
     coro: Coroutine
-        Callback invoked **BEFORE** child callback is invoked.
-        If this returns False (not falsy, False),
+        (Required) Callback invoked when this group is called.
+        (Base groups cannot currently be called,
+        but it will become possible at some point:
+        https://github.com/discord/discord-api-docs/issues/2393)
+    in_addition: :class:`bool`
+        If ``True``, ``coro`` is invoked before a child command is.
+        If ``False``, ``coro`` is only invoked when calling the base group.
     slash: Mapping[:class:`str`, Union[:class:`Group`, :class:`Command`]]
+        Subcommands of this group.
     """
-    def __init__(self, coro: Coroutine, **kwargs):
-        self.id = None
-        self.name = kwargs.pop('name', coro.__name__)
-        self.description = kwargs.pop('description', coro.__doc__)
-        self.guild_id = kwargs.pop('guild_id', None)
-        self.parent = kwargs.pop('parent', None)
-        self.coro = coro
-        self.slash = {}
+    cog = None
 
-    @property
-    def qualname(self):
-        """Fully qualified name of command, including group names."""
-        if self.parent is None:
-            return self.name
-        return self.parent.qualname + ' ' + self.name
+    def __init__(self, coro: Coroutine, in_addition=False, **kwargs):
+        super().__init__(coro, **kwargs)
+        self.in_addition = in_addition
+        self.slash = {}
 
     def slash_cmd(self, **kwargs):
         """See :class:`Command` doc"""
         kwargs['parent'] = self
         def decorator(func):
             cmd = Command(func, **kwargs)
+            cmd.cog = self.cog
             self.slash[cmd.name] = cmd
             return cmd
         return decorator
 
     def add_slash(self, func, **kwargs):
         """See :class:`Command` doc"""
-        kwargs['parent'] = self
-        cmd = Command(func, **kwargs)
-        self.slash[cmd.name] = cmd
+        self.slash_cmd(**kwargs)(func)
 
     def slash_group(self, **kwargs):
         """See :class:`Group` doc"""
         kwargs['parent'] = self
         def decorator(func):
             group = Group(func, **kwargs)
+            group.cog = self.cog
             self.slash[group.name] = group
             return group
         return decorator
 
     def add_slash_group(self, func, **kwargs):
         """See :class:`Group` doc"""
-        kwargs['parent'] = self
-        group = Group(func, **kwargs)
-        self.slash[group.name] = group
+        self.slash_group(**kwargs)(func)
 
     def to_dict(self):
         data = {
@@ -547,15 +635,15 @@ def group(**kwargs):
         return Group(func, **kwargs)
     return decorator
 
-logger = logging.getLogger('discord.ext.status')
+logger = logging.getLogger('discord.ext.slash')
 logger.setLevel(logging.INFO)
 
 class SlashBot(commands.Bot):
     """A bot that supports slash commands."""
 
     def __init__(self, *args, **kwargs):
-        self.debug_guild = kwargs.pop('debug_guild', None)
         super().__init__(*args, **kwargs)
+        self.debug_guild = int(kwargs.pop('debug_guild', 0)) or None
         self.slash = set()
         @self.listen()
         async def on_ready():
@@ -580,7 +668,7 @@ class SlashBot(commands.Bot):
 
     def add_slash(self, func, **kwargs):
         """See :class:`Command` doc"""
-        self.slash.add(Command(func, **kwargs))
+        self.slash_cmd(**kwargs)(func)
 
     def slash_group(self, **kwargs):
         """See :class:`Group` doc"""
@@ -592,17 +680,18 @@ class SlashBot(commands.Bot):
 
     def add_slash_group(self, func, **kwargs):
         """See :class:`Group` doc"""
-        group = Group(func, **kwargs)
-        self.slash.add(group)
+        self.slash_group(**kwargs)(func)
 
     def add_slash_cog(self, cog):
         """Add all attributes of ``cog`` that are
-        :class:`Group` or :class:`Command` instances.
+        :class:`Command` or :class:`Group` instances.
         """
         for key in dir(cog):
             obj = getattr(cog, key)
             if isinstance(obj, (Group, Command)):
-                self.slash.add(obj)
+                obj.cog = cog
+                if obj.parent is None:
+                    self.slash.add(obj)
 
     async def application_info(self):
         self.app_info = await super().application_info()
@@ -615,7 +704,7 @@ class SlashBot(commands.Bot):
                 ', please open an issue for this: '
                 'https://github.com/Kenny2github/discord-ext-slash/issues/new')
         for maybe_cmd in self.slash:
-            if maybe_cmd.id == event['data']['id']:
+            if maybe_cmd.id == int(event['data']['id']):
                 cmd = maybe_cmd
                 break
         else:
@@ -690,18 +779,22 @@ class SlashBot(commands.Bot):
             cmd_dict['application_id'] = done[name]['application_id']
             if done[name] == cmd_dict:
                 logger.debug('GET\t%s\tin guild\t%s', name, guild_id)
-                todo[name].id = done[name]['id']
+                todo[name].id = int(done[name]['id'])
             else:
                 state['PATCH'].setdefault(guild_id, {})[name] \
-                    = {'json': todo[name].to_dict(), 'id': done[name]['id'],
+                    = {'json': todo[name].to_dict(), 'id': int(done[name]['id']),
                        'cmd': todo[name]}
         for name in to_delete:
             state['DELETE'].setdefault(guild_id, {})[name] \
-                = {'id': done[name]['id']}
+                = {'id': int(done[name]['id'])}
 
     async def process_command(self, name, guild_id, route, kwargs):
         cmd = kwargs.pop('cmd', None)
-        data = await self.http.request(route, **kwargs)
+        try:
+            data = await self.http.request(route, **kwargs)
+        except discord.HTTPException:
+            logger.exception('Error when processing command %s', name)
+            return
         if cmd is not None:
-            cmd.id = data['id']
+            cmd.id = int(data['id'])
         logger.debug('%s\t%s\tin guild\t%s', route.method, name, guild_id)
